@@ -1,26 +1,10 @@
 ####################################################################################################################
-# ' @rdname AQSearch.Parameter
-# ' @name AQSearch.Parameter
-# ' @description This function allow the user to search the LLSR database to find any ATPS that matches the used criteria.
-# ' @export
-####################################################################################################################
-# AQSearch.Parameter <-
-#   function(db = LLSR::llsr_data,
-#            db.CompA = NULL,
-#            db.CompB = NULL,
-#            db.CompC = NULL,
-#            db.Temp = NULL,
-#            db.ph = NULL,
-#            db.uid = NULL,
-#            stacked = FALSE,
-#            ...
-#   )
-#   UseMethod("AQSearch.Parameter")
+# ' @importFrom crayon red bold
 ####################################################################################################################
 #' @rdname AQSearch.Parameter
 #' @title Search function for ATPS Systems data
-#' @description This function allow the user to search the package database to find any ATPS that matches the used criteria.
-#' @details The function return the systems that matches the criteria submit by the user. 
+#' @description This function allow the user to search the package database to find any ATPS that matches the available criteria.
+#' @details The function return the systems that matches the criteria submitted by the user.
 #' @param db A highly structure db containing data from previously analised data. LLSR database is used by default but user may input his own db if formatted properly.
 #' @param db.CompA A String variable containing either the CAS, chemical formula or name of the upper phase enriched component..
 #' @param db.CompB A String variable containing either the CAS, chemical formula or name of the lower phase component.
@@ -62,45 +46,48 @@ AQSearch.Parameter <-
     for (modelName in AQSysList()){
       # output variable is initialised with data from db.
       db.grep <- db$db.sys[[modelName]]
-      if (!is.null(db.CompA)) {
+      #
+      if ((nrow(db.grep)==0)&(ncol(db.grep)==0)) {db.ph = db.CompA = db.CompB = db.Temp = db.CompC = db.uid = NULL}
+      #
+      if (is.valid(db.CompA)) {
         db.CompA.names <- db$db.cas[grep(tolower(db.CompA), tolower(db$db.cas$CAS.NAME), fixed = TRUE), "CAS.NAME"]
         db.CompA.altNames <- db$db.cas[grep(tolower(db.CompA), tolower(db$db.cas$CAS.COMMON), fixed = TRUE), "CAS.NAME"]
         db.CompA.cas <- db$db.cas[grep(tolower(db.CompA), tolower(db$db.cas$CAS.CODE), fixed = TRUE), "CAS.NAME"]
         #
-        db.chem.names <- c(db.CompA.names, db.CompA.altNames, db.CompA.cas)
+        db.chem.names <- list(db.CompA.names, db.CompA.altNames, db.CompA.cas)
         db.chem.names <- ifelse(length(db.chem.names) == 0, "", db.chem.names)
-        db.grep <- db.grep[which(db.chem.names == db.grep, TRUE)[, "row"],]
+        db.grep <- db.grep[matchComp(db.chem.names, db.grep), ]
       }
       # search a system that matchs the lower-phase component, if search parameter is not null.
-      if (!is.null(db.CompB)) {
+      if (is.valid(db.CompB)) {
         db.CompB.names <- db$db.cas[grep(tolower(db.CompB), tolower(db$db.cas$CAS.NAME), fixed = TRUE), "CAS.NAME"]
         db.CompB.altNames <- db$db.cas[grep(tolower(db.CompB), tolower(db$db.cas$CAS.COMMON), fixed = TRUE), "CAS.NAME"]
         db.CompB.cas <- db$db.cas[grep(tolower(db.CompB), tolower(db$db.cas$CAS.CODE), fixed = TRUE), "CAS.NAME"]
         #
-        db.chem.names <- c(db.CompB.names, db.CompB.altNames, db.CompB.cas)
+        db.chem.names <- list(db.CompB.names, db.CompB.altNames, db.CompB.cas)
         db.chem.names <- ifelse(length(db.chem.names) == 0, "", db.chem.names)
-        db.grep <- db.grep[which(db.chem.names == db.grep, TRUE)[, "row"],]
+        db.grep <- db.grep[matchComp(db.chem.names, db.grep), ]
       }
       # search a system that matchs the additive component, if search parameter is not null.
-      if (!is.null(db.CompC)) {
+      if (is.valid(db.CompC)) {
         db.CompC.names <- db$db.cas[grep(tolower(db.CompC), tolower(db$db.cas$CAS.NAME), fixed = TRUE), "CAS.NAME"]
         db.CompC.altNames <- db$db.cas[grep(tolower(db.CompC), tolower(db$db.cas$CAS.COMMON), fixed = TRUE), "CAS.NAME"]
         db.CompC.cas <- db$db.cas[grep(tolower(db.CompC), tolower(db$db.cas$CAS.CODE), fixed = TRUE), "CAS.NAME"]
         #
-        db.chem.names <- c(db.CompC.names, db.CompC.altNames, db.CompC.cas)
+        db.chem.names <- list(db.CompC.names, db.CompC.altNames, db.CompC.cas)
         db.chem.names <- ifelse(length(db.chem.names) == 0, "", db.chem.names)
-        db.grep <- db.grep[which(db.chem.names == db.grep, TRUE)[, "row"],]
+        db.grep <- db.grep[matchComp(db.chem.names, db.grep), ]
       }
       # search a system that matchs the system's temperature, if search parameter is not null.
-      if (!is.null(db.Temp)) {
-        db.grep <- db.grep[db.grep$TEMP == db.Temp, ]
+      if (is.valid(db.Temp)) {
+        db.grep <- db.grep[(!is.na(db.grep$TEMP) & db.grep$TEMP == db.Temp), ]
       }
       # search a system that matchs the system's pH, if search parameter is not null.
-      if (!is.null(db.ph)) {
-        db.grep <- db.grep[db.grep$PH == db.ph, ]
+      if (is.valid(db.ph)) {
+        db.grep <- db.grep[(!is.na(db.grep$PH) & db.grep$PH == db.ph), ]
       }
       # search a system that matchs the system's UID, if search parameter is not null.
-      if (!is.null(db.uid)) {
+      if (is.valid(db.uid)) {
         db.grep <- db.grep[matchUID(db.uid, db.grep),]
       }
       #
@@ -111,7 +98,7 @@ AQSearch.Parameter <-
           db.ans[[modelName]] <- db.grep
         }
         #
-        cat(paste("    [", modelName, "] ", "Your search had [", nrow(db.grep), "] results.", "\n", sep = ""))
+        cat(paste("    [", modelName, "] ", "Your search had [", red(nrow(db.grep)), "] results.", "\n", sep = ""))
       } else {
         cat(paste("    [", modelName, "] ", "Your search had no results.", "\n", sep = ""))
       }
@@ -119,7 +106,7 @@ AQSearch.Parameter <-
     #
     # If search isn't null, evaluate data
     if (length(db.ans) != 0)  {
-     invisible(db.ans)
+     invisible(RevREF(db.ans, db))
       #
     }else{
       # Triggers an "no results" error
